package jalview.rest;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import jalview.bin.Cache;
import jalview.rest.RestHandler.EndpointI;

public abstract class AbstractEndpoint implements EndpointI
{
  private final String path;

  private API api;

  private final String name;

  private final String parameters;

  private final String description;

  public AbstractEndpoint(API api, String path, String name,
          String parameters, String description)
  {
    this.api = api;
    this.path = path;
    this.name = name;
    this.parameters = parameters;
    this.description = description;
  }

  public String getPath()
  {
    return this.path;
  }

  protected API getAPI()
  {
    return this.api;
  }

  public String getName()
  {
    return this.name;
  }

  public String getParameters()
  {
    return this.parameters;
  }

  public String getDescription()
  {
    return this.description;
  }

  public abstract void processEndpoint(HttpServletRequest request,
          HttpServletResponse response);

  /*
   * Shared methods below here
   */

  protected String[] getEndpointPathParameters(HttpServletRequest request)
  {
    String pathInfo = request.getPathInfo();
    int slashpos = pathInfo.indexOf('/', 1);
    return slashpos < 1 ? null
            : pathInfo.substring(slashpos + 1).split("/");
  }

  protected void returnError(HttpServletRequest request,
          HttpServletResponse response, String message)
  {
    response.setStatus(500); // set this to something better
    String endpointName = getPath();
    Cache.error(getAPI().getName() + " error: endpoint " + endpointName
            + " failed: '" + message + "'");
    try
    {
      PrintWriter writer = response.getWriter();
      writer.write("Endpoint " + endpointName + ": " + message);
      writer.close();
    } catch (IOException e)
    {
      Cache.debug(e);
    }
  }

  protected String getRequestBody(HttpServletRequest request)
          throws IOException
  {
    StringBuilder sb = new StringBuilder();
    BufferedReader reader = request.getReader();
    try
    {
      String line;
      while ((line = reader.readLine()) != null)
      {
        sb.append(line).append('\n');
      }
    } finally
    {
      reader.close();
    }
    return sb.toString();
  }

  protected boolean checkParameters(HttpServletRequest request,
          HttpServletResponse response, int i)
  {
    String[] parameters = getEndpointPathParameters(request);

    // check we can run fetchsequence
    if (parameters.length < i)
    {
      returnError(request, response,
              "requires parameters:" + getParameters() + "\n" + getName()
                      + ": " + getDescription());
      return false;
    }
    return true;
  }

}