package jalview.rest;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

public class HighlightSequenceEndpoint extends AbstractEndpoint
{
  public HighlightSequenceEndpoint(API api)
  {
    super(api, path, name, parameters, description);
  }

  protected static final String path = "highlight";

  private static final String name = "Highlight positions";

  private static final String parameters = "<sequence names>,<ranges>";

  private static final String description = "Highlight the specified sequences with the specified range(s)";

  public void processEndpoint(HttpServletRequest request,
          HttpServletResponse response)
  {
    if (!checkParameters(request, response, 2))
    {
      return;
    }
    String[] parameters = getEndpointPathParameters(request);

    String rangesString = parameters[0];
    String[] rangeStrings = rangesString.split(",");
    int[][] ranges = new int[rangeStrings.length][2];
    for (int i = 0; i < rangeStrings.length; i++)
    {
      String range = rangeStrings[i];
      try
      {
        int hyphenpos = range.indexOf('-');
        if (hyphenpos < 0)
        {
          ranges[i][0] = Integer.parseInt(range);
          ranges[i][1] = ranges[i][0];
        }
        else
        {
          ranges[i][0] = Integer.parseInt(range.substring(0, hyphenpos));
          ranges[i][1] = Integer.parseInt(range.substring(hyphenpos));
        }
      } catch (NumberFormatException nfe)
      {
        returnError(request, response,
                "couldn't parse ranges component '" + range + "'");
        return;
      }
    }

  }
}